// forceRandomCPU.cpp
// By Joseph K Aicher, Summer 2014 with Professor Russell M Taylor II

#include "forceRandomCPU.h"

// what's the name of this force?
const char * forceRandomCPU::getName( )
{
    return "Force Brownian (CPU)";
}

// constructor
forceRandomCPU::forceRandomCPU(
    std::string sim_CLdir, // the path of directory with our openCL source code
    cl_context sim_context, // OpenCL context passed in from simulation
    cl_command_queue sim_queue, // OpenCL queue passed in from simulation,
    cl_ulong sim_numParticles, // number of particles in the simulation
    cl_ulong sim_totalMaxInteractions, // total maximum number of interactions (per particle) in Jacobian
    CS_REAL sim_timeStep, // the time step in the simulation
    CS_REAL randomForce, // the random force parameter from the configuration file
    int randomSeed // seed being used by the random number generator
    ) : genericForce( sim_CLdir, sim_context, sim_queue, sim_numParticles, sim_totalMaxInteractions, true )
{
    // genericForce constructor has set up the queue, the number of particles, and the time step.

    // start setting theRandomForce by setting to randomForce from configuration.
    theRandomForce = randomForce;

    // seed the random engine
    rEngine.seed( static_cast<unsigned long>( randomSeed ) );

    // set up the number of columns, and set the forces (initially) to zero
    columns = 1; // we only have one source for the random force
    initForcesZero( );
}

// calculating the force
void forceRandomCPU::calculateForce( )
{
    // populate forces with (new) random numbers
    for (cl_ulong i = 0; i < numParticles * columns; ++i) {
        forces[i].s[0] = theRandomForce * rNormal( rEngine );
        forces[i].s[1] = theRandomForce * rNormal( rEngine );
        forces[i].s[2] = theRandomForce * rNormal( rEngine );
    }

    // copy to the random force buffer
    updateDeviceForces( );
}