// forceSpringsCL.h
// By Joseph K Aicher, Summer 2014 with Professor Russell M Taylor II

#include "genericForce.h"
#include "structureLoader.h"

#ifndef FORCESPRINGSCL_H
#define FORCESPRINGSCL_H

typedef struct simSPRING {
    cl_ulong m1, m2; // indices for particles spring attached to
    cl_ulong ndx1, ndx2; // indices for force vector
    CS_REAL restLength;
    CS_REAL springConstant;
    CS_REAL springDrag; // what's the drag constant ( f1 = - springDrag * (v1 - v2).
} simSPRING;

class forceSpringsCL : public genericForce
{
protected :
    // details about the simulation
    cl_ulong numSprings; // the number of springs we have

    // openCL details
    cl_mem clSpring;
    cl_kernel clKernelSpring; // calculates spring forces
    size_t clKernelSpringGlobalSize; // the global size to use with KernelSpring
    cl_kernel clKernelSpringJacobian; // calculates spring forces and Jacobian
    size_t clKernelSpringJacobianGlobalSize; // the global size to use with clKernelSpringJacobian
    cl_kernel clKernelGetDiagonal; // takes results from clKernelSpringJacobian to populate the diagonal
    size_t clKernelGetDiagonalGlobalSize;

public :
    // constructor
    forceSpringsCL (
        std::string sim_CLdir, // the path of directory with our openCL source code
        cl_context sim_context, // OpenCL context passed in from simulation
        cl_command_queue sim_queue, // OpenCL queue passed in from simulation,
        cl_ulong sim_numParticles, // number of particles in the simulation
        cl_ulong sim_totalMaxInteractions, // total maximum number of interactions (per particle) in Jacobian
        cl_device_id sim_deviceId, // the deviceId we are running on
        cl_mem sim_clPosition, // the openCL pointer for the particles' positions
        cl_mem sim_clVelocity, // the openCL pointer for the particles' velocities
        SPRING_node * sh, // the spring nodes from the configuration (MassMesh)
        CS_REAL springDamping // the springDrag = springDamping * springConstant (same definition as in SOFA)
        );

    // destructor
    virtual ~forceSpringsCL ( );

    // updates clForce with spring forces
    virtual void calculateForce ( );

    // updates clForce and clJacobian
    virtual void calculateForceAndJacobian ( );
    
    // gets the name of the force
    virtual const char * getName ( );
};


#endif
