// solverIMEXEuler.h
// By Joseph K Aicher, Summer 2014 with Professor Russell M Taylor II

#include "genericSolver.h"

#ifndef SOLVERIMEXEULER_H
#define SOLVERIMEXEULER_H

class solverIMEXEuler : public genericSolver
{
protected :
    // implementation details of this solver

    // integration kernel
    cl_kernel clKernelIntegrate; // integrates position and velocity by one time step
    size_t clKernelIntegrateGlobalSize; // global size corresponding to this kernel

    // openCL objects for the time-scaled inverse mass and the "damping scale"
    // recall that IMEX Euler is based on the update
    // m*(v_{t+dt}-v_{t})= - m * damping * v_{t+dt} + F(x_{t},t)
    // (assuming that damping is still dependent on mass to get the drag instead of just drag)
    // then v_{t+dt} = (1 / (1 + damping * dt)) * ( v_{t} + (dt / m) * F ).
    // inverse mass is dt/m, damping scale is (1/(1+damping*dt))
    cl_mem clInverseMass; // the (scaled) (inverse) masses of our objects
    cl_mem clDampingScale; // the (scaled) (inverse) damping of our objects

    

public :
    // constructor
    solverIMEXEuler (
        std::string sim_CLdir, // the path of directory with our openCL source code
        cl_context sim_context, // OpenCL context passed in from simulation
        cl_command_queue sim_queue, // OpenCL queue passed in from simulation
        CS_REAL sim_timeStep, // the time step of the simulation
        CS_REAL3 * sim_position, // the initial configuration of positions
        cl_ulong sim_numParticles, // the number of particles in the simulation
        cl_device_id sim_deviceId, // the device id to create the program on
        CS_REAL * mass, // the array of mass values
        CS_REAL * massDamping // the array of mass damping values
        );

    // destructor
    virtual ~solverIMEXEuler ( );

    // attach forces to this
    virtual void attachForces ( 
        sumForces * totalForce
        );

    // integrate forward by one time step
    virtual void integrateStep ( );

    // what's the name of this solver
    virtual const char * getName ( );


};


#endif